@@ -1,4 +1,7 @@
+import bz2
 import csv
+import gzip
+import lzma
 import os
 import shutil
 import sys
@@ -6,39 +9,35 @@ import uuid
 from subprocess import Popen
 
 import pandas as pd
+import py7zr
 from pyunpack import Archive
 from tika import parser
-import py7zr
-import bz2
-import gzip
-import lzma
 
 from texta_parsers.email_parser import EmailParser
-from texta_parsers.settings import META_FIELD, FACE_VECTOR_FIELD_NAME
+from texta_parsers.settings import ES_URL, FACE_VECTOR_FIELD_NAME, META_FIELD
+from texta_parsers.tools import rar_extractor, tar_extractor, utils
 from texta_parsers.tools.extension import Extension
 from texta_parsers.tools.scanner import DocScanner
-from texta_parsers.tools import rar_extractor
-from texta_parsers.tools import tar_extractor
-from texta_parsers.tools import utils
 from . import exceptions
-
-
 # for digidoc parser
+from .tools.elastic import ESImporter
+
+
 FNULL = open(os.devnull, "w")
 
 
 class DocParser:
-    
-    def __init__(self, 
-        save_attachments=False,
-        save_mails=False,
-        save_documents=False,
-        parse_attachments=True,
-        save_path = "parsed_files",
-        temp_dir="", 
-        languages=["est", "eng", "rus"],
-        max_file_size = 100,
-        ignore_digidoc = False):
+
+    def __init__(self,
+                 save_attachments=False,
+                 save_mails=False,
+                 save_documents=False,
+                 parse_attachments=True,
+                 save_path="parsed_files",
+                 temp_dir="",
+                 languages=["est", "eng", "rus"],
+                 max_file_size=100,
+                 ignore_digidoc=False):
 
         """
         :param: bool save_attachments: Whether to save email attachments.
@@ -52,7 +51,6 @@ class DocParser:
         :param: [bool] ignore digidoc: Whether digidoc containers should be ignored.
         """
 
-        
         self.save_attachments = save_attachments
         self.save_mails = save_mails
         self.save_documents = save_documents
@@ -64,7 +62,7 @@ class DocParser:
         self.ignore_digidoc = ignore_digidoc
         self.scanner = DocScanner()
 
-        if(self.ignore_digidoc == False):
+        if (self.ignore_digidoc == False):
             if not utils.check_digidoc_exists():
                 raise FileNotFoundError("Digidoc-tool not found. Either set ignore_digidoc=True or install the tool.")
 
@@ -116,29 +114,34 @@ class DocParser:
     def _extract_rar(self, input_path, output_dir):
         rar_extractor.extract_rar_file(input_path, output_dir)
 
+
     def _extract_tar(self, input_path, output_dir):
         tar_extractor.extract_tar_file(input_path, output_dir)
 
+
     def _extract_7z(self, input_path, output_path):
         with py7zr.SevenZipFile(input_path, mode='r') as z:
             z.extractall(path=output_path)
 
+
     def _extract_bz2(self, input_path, output_path):
         output_loc = os.path.join(output_path, "out.decompressed")
         with open(output_loc, 'wb') as out_file, bz2.BZ2File(input_path, 'rb') as in_file:
-            for data in iter(lambda : in_file.read(100 * 1024), b''):
+            for data in iter(lambda: in_file.read(100 * 1024), b''):
                 out_file.write(data)
 
+
     def _extract_gz(self, input_path, output_path):
         output_loc = os.path.join(output_path, "out.decompressed")
         with open(output_loc, 'wb') as out_file, gzip.GzipFile(input_path, 'rb') as in_file:
-            for data in iter(lambda : in_file.read(100 * 1024), b''):
+            for data in iter(lambda: in_file.read(100 * 1024), b''):
                 out_file.write(data)
 
+
     def _extract_xz(self, input_path, output_path):
         output_loc = os.path.join(output_path, "out.decompressed")
         with open(output_loc, 'wb') as out_file, lzma.LZMAFile(input_path, 'rb') as in_file:
-            for data in iter(lambda : in_file.read(100 * 1024), b''):
+            for data in iter(lambda: in_file.read(100 * 1024), b''):
                 out_file.write(data)
 
 
@@ -160,9 +163,9 @@ class DocParser:
             self._extract_tar(input_path, output_dir)
         elif extension == ".7z":
             self._extract_7z(input_path, output_dir)
-        #Note: as the name of the file in the bz2 archive is unknow, 
-        #extension prediction relies entirely on magic, which tends to fail
-        #in some cases (e.g digidoc)
+        # Note: as the name of the file in the bz2 archive is unknow,
+        # extension prediction relies entirely on magic, which tends to fail
+        # in some cases (e.g digidoc)
         elif extension == ".bz2":
             self._extract_bz2(input_path, output_dir)
         elif extension == ".gz":
@@ -196,8 +199,9 @@ class DocParser:
         if os.path.exists(self.temp_dir):
             shutil.rmtree(self.temp_dir)
 
+
     @staticmethod
-    def _parse_file(document, languages):
+    def _parse_file(document, languages, use_face_analyzer=False):
         """
         Parses document using TIKA (for everything) and FaceAnalyzer (for images).
         """
@@ -208,13 +212,17 @@ class DocParser:
                 "X-Tika-PDFextractInlineImages": "true"
             }
         }
+
         # extract faces if an image
-        if document["extension"] in Extension.IMAGE_EXTENSIONS:
+        can_face_analyzer = ESImporter.check_face_analyzer_elegibility(ES_URL)
+        is_image_file = document["extension"] in Extension.IMAGE_EXTENSIONS
+        if use_face_analyzer and is_image_file and can_face_analyzer:
             # import here so we won't die on init if analyzer not installed
             from texta_face_analyzer.face_analyzer import FaceAnalyzer
             face_vectors = FaceAnalyzer().vectorize_photo(document["path"])
             if face_vectors:
                 output_document[FACE_VECTOR_FIELD_NAME] = [{"name": "UNKOWN_FACE", "vector": v} for v in face_vectors]
+
         # extract text using TIKA
         tika_output = parser.from_file(document["path"], requestOptions=tika_options)
         content = tika_output["content"]
@@ -227,9 +235,10 @@ class DocParser:
         # return as list
         return [output_document]
 
+
     @staticmethod
     def _file_size_ok(max_file_size, document):
-        if(os.path.getsize(document["path"]) > max_file_size*1024**2):
+        if (os.path.getsize(document["path"]) > max_file_size * 1024 ** 2):
             return False
         return True
 
@@ -269,69 +278,73 @@ class DocParser:
                 for _, row in reader.iterrows():
                     yield row.to_dict()
 
+
     def save_attachment(self, attachment):
-        if(self.save_attachments == True):
+        if (self.save_attachments == True):
             save_path = os.path.join(self.save_path, "attachments", attachment["parent_id"])
 
             if not os.path.exists(save_path):
                 os.makedirs(save_path)
 
-            #generate random filename for attachment if for some reason it is missing
-            if(attachment["filename"] == ""):
+            # generate random filename for attachment if for some reason it is missing
+            if (attachment["filename"] == ""):
                 filename = os.path.join(save_path, uuid.uuid4().hex + attachment[META_FIELD]["extension"])
             else:
-                filename =  os.path.join(save_path, attachment["filename"])
-            
+                filename = os.path.join(save_path, attachment["filename"])
+
             shutil.copyfile(attachment[META_FIELD]["path"], filename)
             attachment[META_FIELD]["location"] = filename
         return True
 
+
     def save_document(self, meta):
-        if(self.save_documents == True):
+        if (self.save_documents == True):
             extension = meta["extension"]
             save_path = os.path.join(self.save_path, "files", extension[1:])
 
             if not os.path.exists(save_path):
                 os.makedirs(save_path)
 
-            filename =  os.path.join(save_path, uuid.uuid4().hex + extension)
+            filename = os.path.join(save_path, uuid.uuid4().hex + extension)
             shutil.copyfile(meta["path"], filename)
             meta["location"] = filename
         return True
 
+
     def _parse_attachment(self, attachment):
-        parsed_attachment = list(self._parse(attachment["content"], attachment["filename"], save_to_file=False)) #this could be a list of multiple files
-        
+        parsed_attachment = list(self._parse(attachment["content"], attachment["filename"], save_to_file=False))  # this could be a list of multiple files
+
         extension = Extension.predict(attachment["content"], file_name=attachment["filename"])
         is_archive = extension in Extension.ARCHIVE_EXTENSIONS
 
         extracted_attachments = []
         for ix, sub_file in enumerate(parsed_attachment):
-            #replicate the fields of original attachment to sub attachment
+            # replicate the fields of original attachment to sub attachment
             sub_attachment = attachment.copy()
             sub_attachment["content"] = sub_file["text"]
 
-            #infer filename from teporal path since the original was extension
-            if(is_archive):
+            # infer filename from teporal path since the original was extension
+            if (is_archive):
                 sub_attachment["filename"] = os.path.split(sub_file["properties"]["path"])[1]
 
-            #also add metadata of the file to the dictionary
+            # also add metadata of the file to the dictionary
             sub_attachment[META_FIELD] = sub_file[META_FIELD]
 
-            #also add potential error messages
-            if("errors" in sub_file):
+            # also add potential error messages
+            if ("errors" in sub_file):
                 sub_attachment["errors"] = sub_file["errors"]
 
             # in case the attachment was an archive and contained multiple files,
             # add index number to attachment id to show it 
-            if(len(parsed_attachment) > 1):
-               sub_attachment["id"] = sub_attachment["id"] + "_" + str(ix+1)
+            if (len(parsed_attachment) > 1):
+                sub_attachment["id"] = sub_attachment["id"] + "_" + str(ix + 1)
 
             extracted_attachments.append(sub_attachment)
-            
+
         return extracted_attachments
 
-    def _parse(self, parser_input, file_name=None, save_to_file=True):
+
+    def _parse(self, parser_input, file_name=None, save_to_file=True, use_face_analyzer=False):
         """
         :param: str parser_input: Base64 string or file path.
         :param: bool save_to_file: As this function is recursive, specifies whether the file
@@ -358,9 +371,9 @@ class DocParser:
             # guess extension (it also performs check if extension is known)
             extension = Extension.predict(file_path, file_name=file_name)
 
-            if(self.ignore_digidoc == True and extension in Extension.DIGIDOC_EXTENSIONS):
+            if (self.ignore_digidoc == True and extension in Extension.DIGIDOC_EXTENSIONS):
                 continue
-            
+
             # in case of an arcive, extract all files from it
             if extension in Extension.ARCHIVE_EXTENSIONS:
                 docs_to_parse = list(self._extract_archive(file_path, self.temp_dir, extension))
@@ -369,20 +382,20 @@ class DocParser:
 
             for doc in docs_to_parse:
                 doc["origin"] = file_name if file_name != None else doc["path"]
-                
+
             # parse all files
             for meta in docs_to_parse:
-                if(not self._file_size_ok(self.max_file_size, meta)):
+                if (not self._file_size_ok(self.max_file_size, meta)):
                     meta["errors"] = "File too large for parsing."
                     yield {META_FIELD: meta}
 
-                #email parser handles all the errors and logs uncaught
+                # email parser handles all the errors and logs uncaught
                 elif (meta["extension"] in Extension.EMAIL_EXTENSIONS):
                     parser = EmailParser(tmp_folder=self.temp_dir, save_path=self.save_path)
                     generator = parser.parse(meta["path"], save_mails=self.save_mails)
 
                     for msg_dict, attachment_dicts in generator:
-                        msg_dict[META_FIELD] = meta.copy() # add metadata to email dictionary
+                        msg_dict[META_FIELD] = meta.copy()  # add metadata to email dictionary
 
                         # attachments in attachment_dicts are not parsed and the field content only contains raw payload in bytes.
                         # thus we need to call parse function on each content to get the text.
@@ -390,11 +403,11 @@ class DocParser:
 
                         extracted_attachments = []
                         for attachment in attachment_dicts:
-                            if("subject" in attachment): #is actually mail, do not parse it (already parsed!)
+                            if ("subject" in attachment):  # is actually mail, do not parse it (already parsed!)
                                 attachment[META_FIELD] = meta.copy()
                                 extracted_attachments.append(attachment)
                             else:
-                                if(self.parse_attachments == True):
+                                if (self.parse_attachments == True):
                                     try:
                                         extracted_attachments += self._parse_attachment(attachment)
                                     except:
@@ -403,26 +416,24 @@ class DocParser:
                                         attachment[META_FIELD]["errors"] = "{}: {}".format(exc_type.__name__, value)
                                         extracted_attachments += [attachment]
 
-
                                     # save attachments if specified
                                     for attachment in extracted_attachments:
-                                        if("subject" not in attachment): #is actually mail, do not save it (already in email_parser if specified so!)
+                                        if ("subject" not in attachment):  # is actually mail, do not save it (already in email_parser if specified so!)
                                             self.save_attachment(attachment)
 
                                 else:
                                     for attachment in attachment_dicts:
                                         attachment[META_FIELD] = meta.copy()
-                                        if("subject" not in attachment):
-                                            if("content" in attachment):
+                                        if ("subject" not in attachment):
+                                            if ("content" in attachment):
                                                 attachment.pop("content")
                                             self.save_attachment(attachment)
 
-
                         yield msg_dict, extracted_attachments if self.parse_attachments else attachment_dicts
 
                 # parse collections
                 elif (meta["extension"] in Extension.COLLECTION_EXTENSIONS):
-                    if(save_to_file):
+                    if (save_to_file):
                         self.save_document(meta)
                     generator = self._parse_collection(meta)
                     try:
@@ -437,10 +448,10 @@ class DocParser:
 
                 # parse everything else
                 else:
-                    if(save_to_file):
+                    if (save_to_file):
                         self.save_document(meta)
                     try:
-                        for parsed_document in self._parse_file(meta, languages=self.langs):
+                        for parsed_document in self._parse_file(meta, languages=self.langs, use_face_analyzer=use_face_analyzer):
                             parsed_document[META_FIELD] = meta.copy()
                             yield parsed_document
                     except:
@@ -449,8 +460,9 @@ class DocParser:
                         meta["errors"] = error_msg
                         yield {META_FIELD: meta}
 
+
     def _remove_temp_path_ref(self, item):
-        if(type(item) == tuple):
+        if (type(item) == tuple):
             item[0][META_FIELD].pop("path")
             for t in item[1]:
                 t[META_FIELD].pop("path")
@@ -458,12 +470,10 @@ class DocParser:
             item[META_FIELD].pop("path")
 
 
-    def parse(self, parser_input, file_name=None):
+    def parse(self, parser_input, file_name=None, use_face_analyzer=False):
         self.temp_dir = self.create_temp_dir_for_parse()
-        generator = self._parse(parser_input, file_name)
+        generator = self._parse(parser_input, file_name, use_face_analyzer=use_face_analyzer)
         for item in generator:
             self._remove_temp_path_ref(item)
             yield item
         self.remove_temp_dir()
-
-                    
