@@ -3,12 +3,13 @@ import os
 import warnings
 
 import elasticsearch
+import elasticsearch_dsl
 from elasticsearch.helpers import bulk
 from elasticsearch_dsl import Keyword, Long, Mapping, Nested, DenseVector
 
 from . import utils
 from .utils import ParserOutputType
-from ..settings import META_FIELD, FACE_VECTOR_FIELD_NAME
+from ..settings import ES_URL, META_FIELD, FACE_VECTOR_FIELD_NAME
 
 
 logging.basicConfig(
@@ -30,6 +31,31 @@ class ESImporter:
         self.es = elasticsearch.Elasticsearch(elastic_url)
 
 
+    @staticmethod
+    def build_flavor(elastic_url: str) -> str:
+        es = elasticsearch.Elasticsearch(elastic_url)
+        return es.info()["version"]["build_flavor"]
+
+
+    @staticmethod
+    def version_number(elastic_url: str) -> int:
+        es = elasticsearch.Elasticsearch(elastic_url)
+        number_string = es.info()["version"]["number"]
+        first, second, third = number_string.split(".")
+        main_version = int(first)
+        return main_version
+
+
+    @staticmethod
+    def check_face_analyzer_elegibility(elastic_url: str, minimum_licence="oss") -> bool:
+        version_number = ESImporter.version_number(elastic_url)
+        build_flavor = ESImporter.build_flavor(elastic_url)
+        if version_number >= 7 and build_flavor != minimum_licence:
+            return True
+        else:
+            return False
+
+
     def _importer_generator(self, generator, index_name: str):
         """
         Used to generate the payload for the Bulk API of the Python Elasticsearch Clients
@@ -40,6 +66,7 @@ class ESImporter:
         for item in generator:
             yield {
                 "_index": index_name,
+                "_type": "_doc",
                 "_source": item
             }
 
@@ -101,6 +128,7 @@ class ESImporter:
 
                 yield {
                     "_index": mails_index,
+                    "_type": "_doc",
                     "_source": mail
                 }
 
@@ -113,6 +141,7 @@ class ESImporter:
                     self._try_add_file_url(attachment, toolkit_url, tk_media_folder)
                     yield {
                         "_index": attachments_index,
+                        "_type": "_doc",
                         "_source": attachment
                     }
             else:
@@ -124,10 +153,35 @@ class ESImporter:
 
                 yield {
                     "_index": index_name,
+                    "_type": "_doc",
                     "_source": item
                 }
 
 
+    def parse_doc_type_from_mapping(self, mapping: dict, default_es7_doctype="_doc") -> str:
+        """
+        Parse the result of the indexes _mapping endpoint to fetch the set
+        doc_type and if it doesn't exist yet (fresh index) put it into the default _doc.
+        From the start of ES6, only a single doc_type is allowed per index.
+        """
+        doc_types = []
+
+        for index, mappings in mapping.items():
+            mappings = mappings["mappings"]
+            for doc_type in mappings:
+                if doc_type != "properties":
+                    doc_types.append(doc_type)
+
+        return doc_types[0] if doc_types else default_es7_doctype
+
+
+    def get_doc_type_for_index(self, index: str):
+        index_interface = elasticsearch_dsl.Index(index, using=self.es)
+        mapping = index_interface.get_mapping()
+        doc_type = self.parse_doc_type_from_mapping(mapping)
+        return doc_type
+
+
     def _add_texta_facts_mapping(self, index):
         self.es.indices.create(index, ignore=[400, 404])
         m = Mapping()
@@ -140,23 +194,28 @@ class ESImporter:
                 "num_val": Long(),
             }
         )
+
         # Set the name of the field along with its mapping body
-        m.field("texta_facts", texta_facts)
-        m.save(index=index, using=self.es)
+        mapping = m.field("texta_facts", texta_facts).to_dict()
+        doc_type = self.get_doc_type_for_index(index)
+        self.es.indices.put_mapping(body=mapping, index=index, doc_type=doc_type, include_type_name=True)
 
 
     def _add_texta_face_vectors_mapping(self, index):
-        self.es.indices.create(index, ignore=[400, 404])
-        m = Mapping()
-        texta_face_vectors = Nested(
-            properties={
-                "name": Keyword(),
-                "vector": DenseVector(dims=128),
-            }
-        )
-        # Set the name of the field along with its mapping body
-        m.field(FACE_VECTOR_FIELD_NAME, texta_face_vectors)
-        m.save(index=index, using=self.es)
+        face_analyzer_elegibility = ESImporter.check_face_analyzer_elegibility(ES_URL)
+        if face_analyzer_elegibility:
+            self.es.indices.create(index, ignore=[400, 404])
+            m = Mapping()
+            texta_face_vectors = Nested(
+                properties={
+                    "name": Keyword(),
+                    "vector": DenseVector(dims=128),
+                }
+            )
+            # Set the name of the field along with its mapping body
+            mapping = m.field(FACE_VECTOR_FIELD_NAME, texta_face_vectors).to_dict()
+            doc_type = self.get_doc_type_for_index(index)
+            self.es.indices.put_mapping(body=mapping, index=index, doc_type=doc_type, include_type_name=True)
 
 
     def _get_index_name(self, item):
